# analyze_sotus.py
# Prof. Lee, Mar 2022, updated Mar 20

"""
Analyze SOTUs 1982-2022 (i.e. recent-ish, all-spoken SOTUs)
"""




START_YEAR = 1982 # all spoken from then on
END_YEAR = 2022

import a3


from sotu_dict import SOTU_DICT as all_sotus
import matplotlib.pyplot as plt
import numpy as np


sotu_dict = {}
years = []
labels = []
for year in list(range(START_YEAR, END_YEAR+1)):
    if year in all_sotus:
        sotu_dict[year] = all_sotus[year]
        years.append(year)
        labels.append(str(year)) 

PREZ_BNDARIES=[1988.5, 1992.5,2000.5,2008.5,2016.5, 2020.5]


def my_percent_avg_or_shorter(tokenlist):
    """Version of a3.percent_avg_or_shorter that only needs one argument.
    For use in our plot_data function. """
    return a3.percent_avg_or_shorter(tokenlist, list(sotu_dict.values()))

def plot_data(result_fn, name, filename):
    """result_fn: the name of the function to use; should not take any parameters other than a tokenlikst.

    name [str]: name of function, to be used in the title of the plot.

    filename [str]: name for the output file for the plot. Ex: out.pdf
    """

    results = []
    for year in years:
        if year in sotu_dict:
            result=result_fn(sotu_dict[year])
            results.append(result)
    a3.print_trend(results, labels)


    fig, ax = plt.subplots()

    ax.set_title(name+". Up=simpler. Vertical bars: new president")
    ax.set_xlabel('year')
    ax.set_ylabel(name)

    plt.scatter(years, results,s=2)

    # get trendline
    z = np.polyfit(years, results, 1)
    p = np.poly1d(z)
    plt.plot(years,p(years),"r--")
    for bndary in PREZ_BNDARIES:
        plt.axvline(x=bndary)

    plt.savefig(filename)
    plt.close(filename)

def plot_zipf_data(filename):
    """Plot the results for a3.zipf with n=10, 50, 100, 500."""

    fig, ax = plt.subplots(2,2)
    fig.suptitle('Zipf.  Up=simpler. Vertical bars: new president.')
    # for each n, what matplotlib color to use
    plotcolors = {10:"b", 50: "c", 100: "y", 500: "m"}
    # for each n, what position in the figure to use
    plotposn = {10:ax[0,0], 50:ax[0,1], 100:ax[1,0], 500:ax[1,1]}

    for n in [10, 50, 100, 500]:
        print(" ---- n: "+str(n)+" ----")
        results = []
        for year in years:
            if year in sotu_dict:
                result=a3.zipf(sotu_dict[year], n)
                results.append(result)
        a3.print_trend(results, labels)

        plotposn[n].set_xlabel('year')
        plotposn[n].set_ylabel("Zipf, n="+str(n))

        plotposn[n].scatter(years, results,s=2, label="n="+str(n),color=plotcolors[n])

        # get trendline
        z = np.polyfit(years, results, 1)
        p = np.poly1d(z)
        plotposn[n].plot(years,p(years),plotcolors[n]+"--")

        for bndary in PREZ_BNDARIES:
            plotposn[n].axvline(x=bndary)
    plt.savefig(filename)
    plt.close(filename)


def neg_len(x):
    """Returns -len(x). (So simpler is larger)"""
    return -len(x)




if __name__ == '__main__':

    # Compare against table of lengths at https://www.presidency.ucsb.edu/documents/presidential-documents-archive-guidebook/annual-messages-congress-the-state-the-union-3

    print("##### Negative length #####")
    plot_data(neg_len, "Negative length", "sotu_len.pdf")

    print("##### ##### (reversed) type-token ratio #####")
    plot_data(a3.type_token_ratio, "Type-token ratio", "sotu_ttr.pdf")

    print("##### \% short words #####")
    plot_data(my_percent_avg_or_shorter, "% words <= average length", "sotu_percent_short.pdf")

    print('##### zipf ######')
    plot_zipf_data("sotu_zipf.pdf")