# freq.py
# Prof. Lee, LJL2
# Mar 9, 2022

"""Provides the Freq class for storing the count of a given wordtype"""

from functools import total_ordering # helps with sorting


# Helper
def _nice_class(x):
    """Return compact string expression the type of x"""
    s = str(type(x))
    return s[s.index("'")+1:-2]

@total_ordering
class Freq:
    """
    A list of Freq objects will be sorted by their freq attributes.


    Attributes:
        wordtype [non-empty str]
        freq [non-negative int]
    """
    def __init__(self, wt, f):
        assert type(wt) == str, ("First arg for new Freq object should be str "                      + "but it was "+repr(wt)+" which has class "
                                  +_nice_class(wt))
        assert type(f) == int and f > 0, ("Second arg for new Freq object " +                              "should be non-negative int, but " +
                                          "was "+repr(f))
        self.wordtype = wt
        self.freq = f

    def __eq__(self, other):
        return (isinstance(other, Freq) and self.wordtype == other.wordtype and
                self.freq == other.freq)

    def __lt__(self, other):
        assert isinstance(other, Freq), ("Tried to compare Freq with a " +
                                          _nice_class(other))
        return self.freq < other.freq

    def __repr__(self):
        return "Freq('"+self.wordtype+"', "+str(self.freq)+")"


# BEGIN REMOVE
if __name__ == '__main__':
    lt = [Freq("hi", 5), Freq("there", 10)]
    print(lt)
    print(sorted(lt,reverse=True))


# END REMOVE